---@alias NamedUpgrade { name: string, replace: ReplaceRelation }

--- Appends available upgrades to the given upgrades array.
---@param upgrades NamedUpgrade[]
function getAvailableSpawnerUpgrades(upgrades)
    for _, spawner in pairs(spawnerSystem.entities) do
        -- if spawner.hasUpgradeSpeed then
        --     upgrades[#upgrades + 1] = { hasUpgradeSpeed = spawner.hasUpgradeSpeed }
        -- end

        local entityToSpawn = spawner.canSpawn.entityToSpawn
        if entityToSpawn.score then
            upgrades[#upgrades + 1] = {
                name = "Double " .. spawner.name .. " value",
                replace = {
                    entityToModify = entityToSpawn,
                    replacement = {
                        score = entityToSpawn.score * 2,
                    },
                },
            }
        end

        assert(spawner.odds, "Expected all spawners to have an `odds` field!")
        upgrades[#upgrades + 1] = {
            name = "Double " .. spawner.name .. " frequency",
            replace = {
                entityToModify = spawner,
                replacement = {
                    odds = spawner.odds,
                }
            },
        }
    end
end

--- Appends available upgrades to the given upgrades array.
---@param upgrades NamedUpgrade[]
function getAvailableCartUpgrades(upgrades)
    for _, cart in pairs(cartSystem.entities) do
        upgrades[#upgrades + 1] = {
            name = "10% Lighter Cart",
            replace = {
                entityToModify = cart,
                replacement = {
                    mass = cart.mass * 0.9
                }
            },
        }
        upgrades[#upgrades + 1] = {
            name = "+20% Cart Launch Right",
            replace = {
                entityToModify = cart.baseVelocity,
                replacement = {
                    x = cart.baseVelocity.x * 1.2
                }
            },
        }
        upgrades[#upgrades + 1] = {
            name = "+20% Cart Launch Up",
            replace = {
                entityToModify = cart.baseVelocity,
                replacement = {
                    y = cart.baseVelocity.y * 1.2
                }
            },
        }
    end
end

---@return NamedUpgrade[]
function getAvailableUpgrades()
    local upgrades = {}
    getAvailableSpawnerUpgrades(upgrades)
    getAvailableCartUpgrades(upgrades)
    return upgrades
end
